@@ -18,7 +18,7 @@
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: caption.c,v 1.9 2002/10/22 04:42:40 mschimek Exp $ */
+/* $Id: caption.c,v 1.19 2005/05/25 02:25:49 mschimek Exp $ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -33,10 +33,8 @@
 #include "tables.h"
 #include "vbi.h"
 
-#define printable(c) ((((c) & 0x7F) < 0x20 || ((c) & 0x7F) > 0x7E) ? '.' : ((c) & 0x7F))
 #define elements(array) (sizeof(array) / sizeof(array[0]))
 
-#define XDS_DEBUG(x) /* x */
 #define ITV_DEBUG(x) /* x */
 #define XDS_SEP_DEBUG(x) /* x */
 #define XDS_SEP_DUMP(x) /* x */
@@ -87,45 +85,6 @@ language[8] = {
 #define UNUSED
 #endif
 
-static const char *
-map_type[] UNUSED = {
-	"unknown", "mono", "simulated stereo", "stereo",
-	"stereo surround", "data service", "unknown", "none"
-};
-
-static const char *
-sap_type[] UNUSED = {
-	"unknown", "mono", "video descriptions", "non-program audio",
-	"special effects", "data service", "unknown", "none"
-};
-
-static const char *
-cgmsa[] UNUSED = {
-	"copying permitted",
-	"-",
-	"one generation copy allowed",
-	"no copying permitted"
-};
-
-static const char *
-scrambling[] UNUSED = {
-	"no pseudo-sync pulse",
-	"pseudo-sync pulse on; color striping off",
-	"pseudo-sync pulse on; 2-line color striping on",
-	"pseudo-sync pulse on; 4-line color striping on"
-};
-
-static const char *
-month_names[] UNUSED = {
-	"0?", "Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug",
-	"Sep", "Oct", "Nov", "Dec", "13?", "14?", "15?"
-};
-
-static const char *
-day_names[] UNUSED = {
-	"0?", "Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"
-};
-
 static uint32_t hcrc[128];
 
 static void init_hcrc(void) __attribute__ ((constructor));
@@ -153,14 +112,14 @@ init_hcrc(void)
 }
 
 static int
-xds_strfu(char *d, char *s, int len)
+xds_strfu(signed char *d, const uint8_t *s, int len)
 {
 	int c, neq = 0;
 
 	for (; len > 0 && *s <= 0x20; s++, len--);
 
 	for (; len > 0; s++, len--) {
-		c = MAX((char) 0x20, *s);
+		c = MAX((uint8_t) 0x20, *s);
 		neq |= *d ^ c;
 		*d++ = c;
 	}
@@ -207,8 +166,6 @@ xds_decoder(vbi_decoder *vbi, int _class, int type,
 	switch (_class) {
 	case XDS_CURRENT: /* 0 */
 	case XDS_FUTURE: /* 1 */
-		XDS_DEBUG(printf((class == XDS_CURRENT) ? "Current " : "Next "));
-
 		if (!(vbi->event_mask & (VBI_EVENT_ASPECT | VBI_EVENT_PROG_INFO)))
 			return;
 
@@ -228,14 +185,6 @@ xds_decoder(vbi_decoder *vbi, int _class, int type,
 			hour = buffer[1] & 31;
 			min = buffer[0] & 63;
 
-			XDS_DEBUG(printf("PIN: %d %s %02d:%02d UTC, "
-					 "D=%d L=%d Z=%d T(ape delayed)=%d\n",
-					 day, month_names[month], hour, min,
-					 !!(buffer[1] & 0x20),
-					 !!(buffer[2] & 0x20),
-					 !!(buffer[3] & 0x20),
-					 !!(buffer[3] & 0x10)));
-
 			if (month == 0 || month > 12
 			    || day == 0 || day > 31
 			    || hour > 23 || min > 59)
@@ -281,14 +230,6 @@ xds_decoder(vbi_decoder *vbi, int _class, int type,
 					esec = buffer[4] & 63;
 			}
 
-			XDS_DEBUG(
-				printf("length: %02d:%02d, ", lhour, lmin);
-				printf("elapsed: %02d:%02d", ehour, emin);
-				if (length >= 5)
-					printf(":%02d", esec);
-				printf("\n");
-			)
-
 			if (lmin > 59 || emin > 59 || esec > 59)
 				return;
 
@@ -305,13 +246,6 @@ xds_decoder(vbi_decoder *vbi, int _class, int type,
 			if (length < 2)
 				return;
 
-			XDS_DEBUG(
-				printf("program title: '");
-				for (i = 0; i < length; i++)
-					putchar(printable(buffer[i]));
-				printf("'\n");
-			)
-
 			neq = xds_strfu(pi->title, buffer, length);
 
 			if (!neq) { /* no title change */
@@ -334,15 +268,6 @@ xds_decoder(vbi_decoder *vbi, int _class, int type,
 		{
 			int neq;
 
-			XDS_DEBUG(
-				printf("program type: ");
-				for (i = 0; i < length; i++)
-					printf((i > 0) ? ", %s" : "%s",
-						vbi_prog_type_str_by_id(
-						 VBI_PROG_CLASSF_EIA_608, buffer[i]));
-				printf("\n");
-			)
-
 			neq = (pi->type_classf != VBI_PROG_CLASSF_EIA_608);
 			pi->type_classf = VBI_PROG_CLASSF_EIA_608;
 
@@ -377,28 +302,6 @@ xds_decoder(vbi_decoder *vbi, int _class, int type,
 			if (buffer[1] & 0x20)
 				dlsv |= VBI_RATING_V;
 
-			XDS_DEBUG(
-				printf("program movie rating: %s, tv rating: ",
-					vbi_rating_str_by_id(VBI_RATING_AUTH_MPAA, r));
-				if (buffer[0] & 0x10) {
-					if (buffer[0] & 0x20)
-						puts(vbi_rating_str_by_id(VBI_RATING_AUTH_TV_CA_FR, g));
-					else
-						puts(vbi_rating_str_by_id(VBI_RATING_AUTH_TV_CA_EN, g));
-				} else {
-					printf("%s; ", vbi_rating_str_by_id(VBI_RATING_AUTH_TV_US, g));
-					if (buffer[1] & 0x20)
-						printf("violence; ");
-					if (buffer[1] & 0x10)
-						printf("sexual situations; ");
-					if (buffer[1] & 8)
-						printf("indecent language; ");
-					if (buffer[0] & 0x20)
-						printf("sexually suggestive dialog");
-					putchar('\n');
-				}
-			)
-
 			if ((buffer[0] & 0x08) == 0) {
 				if (r == 0) return;
 				auth = VBI_RATING_AUTH_MPAA;
@@ -456,15 +359,14 @@ xds_decoder(vbi_decoder *vbi, int _class, int type,
 			if (length != 2)
 				return;
 
-			XDS_DEBUG(printf("main audio: %s, %s; "
-					 "second audio program: %s, %s\n",
-					 map_type[buffer[0] & 7], language[(buffer[0] >> 3) & 7],
-					 sap_type[buffer[1] & 7], language[(buffer[1] >> 3) & 7]));
-
 			for (i = 0; i < 2; i++) {
 				int l = (buffer[i] >> 3) & 7;
 				vbi_audio_mode m = mode[i][buffer[i] & 7];
-				char *s = ((1 << l) & 0xC1) ? NULL : language[l];
+				/* should be const char *, but I got that
+				   wrong and cannot change the public
+				   pi->audio[].language type now. */
+				unsigned char *s = ((1 << l) & 0xC1) ? NULL :
+					(unsigned char *) language[l];
 
 				if (pi->audio[i].mode != m) {
 					neq = 1; pi->audio[i].mode = m;
@@ -490,15 +392,15 @@ xds_decoder(vbi_decoder *vbi, int _class, int type,
 			for (i = 0; i < length; i++) {
 				int ch = buffer[i] & 7;
 				int l = (buffer[i] >> 3) & 7;
-				char *s;
+				const char *s;
 
 				ch = (ch & 1) * 4 + (ch >> 1);
 
 				services |= 1 << ch;
 				s = ((1 << l) & 0xC1) ? NULL : language[l];
 
-				if (pi->caption_language[ch] != s) {
-					neq = 1; pi->caption_language[ch] = s;
+				if (pi->caption_language[ch] != (unsigned char *) s) {
+					neq = 1; pi->caption_language[ch] = (unsigned char *) s;
 				}
 
 				if (_class == XDS_CURRENT)
@@ -508,16 +410,6 @@ xds_decoder(vbi_decoder *vbi, int _class, int type,
 
 			xds_intfu(pi->caption_services, services);
 
-			XDS_DEBUG(
-				printf("program caption services:\n");
-				for (i = 0; i < length; i++)
-					printf("Line %3d, channel %d, %s: %s\n",
-						(buffer[i] & 4) ? 284 : 21,
-						(buffer[i] & 2) ? 2 : 1,
-						(buffer[i] & 1) ? "text      " : "captioning",
-						language[(buffer[i] >> 3) & 7]);
-			)
-
 			break;
 		}
 
@@ -525,13 +417,6 @@ xds_decoder(vbi_decoder *vbi, int _class, int type,
 			if (length != 1)
 				return;
 
-			XDS_DEBUG(
-				printf("CGMS: %s", cgmsa[(buffer[0] >> 3) & 3]);
-				if (buffer[0] & 0x18)
-					printf("; %s", scrambling[(buffer[0] >> 1) & 3]);
-				printf("; analog source: %d", buffer[0] & 1);
-			)
-
 			xds_intfu(pi->cgms_a, buffer[0] & 63);
 
 			break;
@@ -555,12 +440,6 @@ xds_decoder(vbi_decoder *vbi, int _class, int type,
 			else
 				r->ratio = 1.0;
 
-			XDS_DEBUG(printf("program aspect ratio info: "
-					 "active start %d, end %d%s\n",
-					 (buffer[0] & 63) + 22, 262 - (buffer[1] & 63),
-					 (length >= 3 && (buffer[2] & 1)) ?
-					 " (anamorphic)" : ""));
-
 			if (memcmp(r, &vbi->prog_info[0].aspect, sizeof(*r)) != 0) {
 				vbi->prog_info[0].aspect = *r;
 				vbi->aspect_source = 3;
@@ -578,21 +457,12 @@ xds_decoder(vbi_decoder *vbi, int _class, int type,
 		{
 			int line = type & 7;
 
-			XDS_DEBUG(
-				printf("program descr. line %d: >", line + 1);
-				for (i = 0; i < length; i++)
-					putchar(printable(buffer[i]));
-				printf("<\n");
-			)
-
 			neq = xds_strfu(pi->description[line], buffer, length);
 
 			break;
 		}
 
 		default:
-			XDS_DEBUG(printf("<unknown %d/%02x length %d>\n",
-					 class, type, length));
 			return; /* no event */
 		}
 
@@ -621,7 +491,7 @@ xds_decoder(vbi_decoder *vbi, int _class, int type,
 			if (xds_strfu(n->name, buffer, length)) {
 				n->cycle = 1;
 			} else if (n->cycle == 1) {
-				char *s = n->name;
+				signed char *s = n->name;
 				uint32_t sum;
 
 				if (n->call[0])
@@ -644,13 +514,6 @@ xds_decoder(vbi_decoder *vbi, int _class, int type,
 				n->cycle = 3;
 			}
 
-			XDS_DEBUG(
-				printf("Network name: '");
-				for (i = 0; i < length; i++)
-					putchar(printable(buffer[i]));
-				printf("'\n");
-			)
-
 			break;
 
 		case 2:		/* network call letters */
@@ -661,13 +524,6 @@ xds_decoder(vbi_decoder *vbi, int _class, int type,
 				}
 			}
 
-			XDS_DEBUG(
-				printf("Network call letters: '");
-				for (i = 0; i < length; i++)
-					putchar(printable(buffer[i]));
-				printf("'\n");
-			)
-
 			break;
 
 		case 3:		/* channel tape delay */
@@ -677,14 +533,9 @@ xds_decoder(vbi_decoder *vbi, int _class, int type,
 			n->tape_delay =
 				(buffer[1] & 31) * 60 + (buffer[0] & 63);
 
-			XDS_DEBUG(printf("Channel tape delay: %02d:%02d",
-					 buffer[1] & 31, buffer[0] & 63));
-
 			break;
 
 		default:
-			XDS_DEBUG(printf("<unknown %d/%02x length %d>\n",
-					 class, type, length));
 			break;
 		}
 
@@ -695,73 +546,33 @@ xds_decoder(vbi_decoder *vbi, int _class, int type,
 		case 1:		/* time of day */
 			if (length != 6)
 				return;
-			XDS_DEBUG(
-				printf("Time of day (UTC): %s, %d %s %d ",
-					day_names[buffer[4] & 7],
-					buffer[2] & 31, month_names[buffer[3] & 15],
-					1990 + (buffer[5] & 63));
-				printf("%02d:%02d ", buffer[1] & 31, buffer[0] & 63);
-				printf("D(ST)=%d, L(eap day)=%d, "
-					"(Second )Z(ero)=%d, T(ape delayed)=%d\n",
-					!!(buffer[1] & 0x20),
-					!!(buffer[2] & 0x20),
-					!!(buffer[3] & 0x20),
-					!!(buffer[3] & 0x10));
-			)
 			break;
 
 		case 2:		/* impulse capture id */
 			if (length != 6)
 				return;
-			XDS_DEBUG(
-				printf("Impulse capture id: %d %s ",
-					buffer[2] & 31, month_names[buffer[3] & 15]);
-				printf("%02d:%02d ", buffer[1] & 31, buffer[0] & 63);
-				printf("length %02d:%02d ",
-					buffer[5] & 63, buffer[4] & 63);
-				printf("D=%d, L=%d, Z=%d, T(ape delayed)=%d\n",
-					!!(buffer[1] & 0x20),
-					!!(buffer[2] & 0x20),
-					!!(buffer[3] & 0x20),
-					!!(buffer[3] & 0x10));
-			)
 			break;
 
 		case 3:		/* supplemental data location */
-			XDS_DEBUG(
-				for (i = 0; i < length; i++)
-					printf("Supplemental data: field %d, line %d\n",
-					       !!(buffer[i] & 0x20), buffer[i] & 31);
-			)
 			break;
 
 		case 4:		/* local time zone */
 			if (length != 1)
 				return;
-			XDS_DEBUG(printf("Local time zone: UTC + %d h; D(ST)=%d\n",
-				buffer[0] & 31, !!(buffer[0] & 0x20)));
 			break;
 
 		case 0x40:	/* out-of-band channel number */
 			if (length != 2)
 				return;
-			XDS_DEBUG(
-				i = (buffer[0] & 63) | ((buffer[1] & 63) << 6);
-				printf("Out-of-band channel %d -- ?\n", i);
-			)
 			break;
 
 		default:
-			XDS_DEBUG(printf("<unknown %d/%02x length %d>\n",
-					 class, type, length));
 			break;
 		}
 
 		break;
 
 	default:
-		XDS_DEBUG(printf("<unknown %d/%02x length %d>\n",
-				 class, type, length));
 		break;
 	}
 }
@@ -771,8 +582,8 @@ xds_separator(vbi_decoder *vbi, uint8_t *buf)
 {
 	struct caption *cc = &vbi->cc;
 	xds_sub_packet *sp = cc->curr_sp;
-	int c1 = vbi_parity(buf[0]);
-	int c2 = vbi_parity(buf[1]);
+	int c1 = vbi_unpar8 (buf[0]);
+	int c2 = vbi_unpar8 (buf[1]);
 	unsigned int class, type;
 
 	XDS_SEP_DEBUG(printf("XDS %02x %02x\n", buf[0], buf[1]));
@@ -794,7 +605,7 @@ xds_separator(vbi_decoder *vbi, uint8_t *buf)
 		class = (c1 - 1) >> 1;
 
 		if (class > elements(cc->sub_packet)
-		    || c2 > elements(cc->sub_packet[0])) {
+		    || c2 > (int) elements(cc->sub_packet[0])) {
 			XDS_SEP_DEBUG(printf("XDS ignore packet %d/0x%02x\n", class, c2));
 			cc->curr_sp = NULL;
 			return;
@@ -832,7 +643,7 @@ xds_separator(vbi_decoder *vbi, uint8_t *buf)
 
 			XDS_SEP_DUMP(
 				for (i = 0; i < sp->count - 2; i++)
-					printf("%c", printable(sp->buffer[i]));
+					printf("%c", vbi_printable (sp->buffer[i]));
 				printf(" %d/0x%02x\n", class, type);
 			)
 		}
@@ -879,7 +690,7 @@ itv_separator(vbi_decoder *vbi, struct caption *cc, char c)
 	if (c >= 0x20) {
 		if (c == '<') // s4-nbc omitted CR
 			itv_separator(vbi, cc, 0);
-		else if (cc->itv_count > sizeof(cc->itv_buf) - 2)
+		else if (cc->itv_count > (int) sizeof(cc->itv_buf) - 2)
 			cc->itv_count = 0;
 
 		cc->itv_buf[cc->itv_count++] = c;
@@ -1423,7 +1234,7 @@ vbi_decode_caption(vbi_decoder *vbi, int line, uint8_t *buf)
 	char c1 = buf[0] & 0x7F;
 	int field2 = 1, i;
 
-	pthread_mutex_unlock(&cc->mutex);
+	pthread_mutex_lock(&cc->mutex);
 
 	switch (line) {
 	case 21:	/* NTSC */
@@ -1436,12 +1247,12 @@ vbi_decode_caption(vbi_decoder *vbi, int line, uint8_t *buf)
 
 	case 284:	/* NTSC */
 		CC_DUMP(
-			putchar(printable(buf[0]));
-			putchar(printable(buf[1]));
+			putchar(vbi_printable (buf[0]));
+			putchar(vbi_printable (buf[1]));
 			fflush(stdout);
 		)
 
-		if (vbi_parity(buf[0]) >= 0) {
+		if (vbi_unpar8 (buf[0]) >= 0) {
 			if (c1 == 0) {
 				goto finish;
 			} else if (c1 <= 0x0F) {
@@ -1465,15 +1276,15 @@ vbi_decode_caption(vbi_decoder *vbi, int line, uint8_t *buf)
 		goto finish;
 	}
 
-	if (vbi_parity(buf[0]) < 0) {
+	if (vbi_unpar8 (buf[0]) < 0) {
 		c1 = 127;
 		buf[0] = c1; /* traditional 'bad' glyph, ccfont has */
 		buf[1] = c1; /*  room, design a special glyph? */
 	}
 
 	CC_DUMP(
-		putchar(printable(buf[0]));
-		putchar(printable(buf[1]));
+		putchar(vbi_printable (buf[0]));
+		putchar(vbi_printable (buf[1]));
 		fflush(stdout);
 	)
 
@@ -1487,7 +1298,7 @@ vbi_decode_caption(vbi_decoder *vbi, int line, uint8_t *buf)
 		break; /* XDS field 1?? */
 
 	case 0x10 ... 0x1F:
-		if (vbi_parity(buf[1]) >= 0) {
+		if (vbi_unpar8 (buf[1]) >= 0) {
 			if (!field2
 			    && buf[0] == cc->last[0]
 			    && buf[1] == cc->last[1]) {
@@ -1509,8 +1320,8 @@ vbi_decode_caption(vbi_decoder *vbi, int line, uint8_t *buf)
 
 	default:
 		CC_TEXT_DUMP(
-			putchar(printable(buf[0]));
-			putchar(printable(buf[1]));
+			putchar(vbi_printable (buf[0]));
+			putchar(vbi_printable (buf[1]));
 			fflush(stdout);
 		)
 
@@ -1539,7 +1350,7 @@ vbi_decode_caption(vbi_decoder *vbi, int line, uint8_t *buf)
 		c = ch->attr;
 
 		for (i = 0; i < 2; i++) {
-			char ci = vbi_parity(buf[i]) & 0x7F; /* 127 if bad */
+			char ci = vbi_unpar8 (buf[i]) & 0x7F; /* 127 if bad */
 
 			if (ci <= 0x1F) /* 0x00 no char, 0x01 ... 0x1F invalid */
 				continue;
