@@ -21,13 +21,16 @@
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: export.c,v 1.13 2002/10/22 04:40:56 mschimek Exp $ */
+/* $Id: export.c,v 1.22 2005/02/17 10:21:16 mschimek Exp $ */
 
-#include "../config.h"
+#ifdef HAVE_CONFIG_H
+#  include "config.h"
+#endif
 
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
+#include <stdarg.h>
 #include <errno.h>
 #include <ctype.h>
 #include <sys/stat.h>
@@ -36,11 +39,11 @@
 #include "export.h"
 #include "vbi.h" /* vbi_asprintf */
 
-const char _zvbi_intl_domainname[] = PACKAGE;
+extern const char _zvbi_intl_domainname[];
 
 /**
  * @addtogroup Export Exporting formatted Teletext and Closed Caption pages
- * @ingroup Service
+ * @ingroup HiDec
  * 
  * Once libzvbi received, decoded and formatted a Teletext or Closed
  * Caption page you will want to render it on screen, print it as
@@ -524,7 +527,7 @@ option_string(vbi_export *e, const char *s2)
 vbi_export *
 vbi_export_new(const char *keyword, char **errstr)
 {
-	unsigned char key[256];
+	char key[256];
 	vbi_export_class *xc;
 	vbi_export *e;
 	unsigned int keylen;
@@ -735,7 +738,7 @@ vbi_export_option_get(vbi_export *export, const char *keyword,
 /**
  * @param export Pointer to a initialized vbi_export object.
  * @param keyword Keyword identifying the option, as in vbi_option_info.
- * @param Varargs New value to set.
+ * @param ... New value to set.
  *
  * Sets the value of the named option. Make sure the value is casted
  * to the correct type (int, double, char *).
@@ -985,8 +988,9 @@ vbi_export_file(vbi_export *export, const char *name,
 	reset_error(export);
 
 	if (!(fp = fopen(name, "w"))) {
-		vbi_export_error_printf(export, _("Cannot create file '%s': Error %d, %s."),
-					name, errno, strerror(errno));
+		vbi_export_error_printf(export,
+					_("Cannot create file '%s': %s."),
+					name, strerror(errno));
 		return FALSE;
 	}
 
@@ -1124,14 +1128,14 @@ vbi_export_invalid_option(vbi_export *export, const char *keyword, ...)
 		case VBI_OPTION_STRING:
 			s = va_arg(args, char *);
 			if (s == NULL)
-				strncpy(buf, "NULL", 4);
+				strcpy(buf, "NULL");
 			else
 				snprintf(buf, sizeof(buf) - 1, "'%s'", s);
 			break;
 		default:
 			fprintf(stderr, "%s: unknown export option type %d\n",
 				__PRETTY_FUNCTION__, oi->type);
-			strncpy(buf, "?", 1);
+			strcpy(buf, "?");
 			break;
 		}
 
